from .base import Collect
from .products import product_main
from .diary import diary_main
from .transaction import transaction_main
from .html import html_main
from .parse_diary import parse_diary_main



def main():
    c = raw_input('get product(p) or diary(d) or transaction(t) or html(h) or parse diary(pd):  \np/d/t/h/pd? ')
    if c.lower()=='p':
        product_main()
    elif c.lower()=='d':
        diary_main()
    elif c.lower()=='t':
        transaction_main()
    elif c.lower()=='h':
        html_main()
    elif c.lower()=='pd':
        parse_diary_main()



# coding: utf-8

import sys
import os
try:
    reload(sys)
    sys.setdefaultencoding("utf-8")
except:
    print('python3')
    
import requests
import json
from json2df import flatten_dict
import pandas as pd

from time import time, ctime, strftime, localtime
from multiprocessing import cpu_count
from multiprocessing.dummy import Pool as ThreadPool


class Transaction(object):
    def __init__(self,  PID):
        self.link = link.format(PID, '{}')
        self.index=1
        self.content = []
        self.pid=PID
        
    def initial(self):
        if self.index==1:
            try:
                r = requests.get(self.link.format(self.index))
                d = json.loads(r.text)
                
                self.page, self.all = int(d['pages']['all']), int(d['pages']['all_num'])
                c = d['list']
                
                if len(c)>0:
                    self.content = self.content+c
                    self.index+= 1   
                    
                print 'page:',self.page,'all:', self.all,
                print '{}/{}'.format(self.index-1,self.page),
            except Exception as e:
                print 'N', 
                try:
                    if self.all==0:
                        print '0',
                    
                #print e
                except:
                    self.initial()
        
    def process(self):
        if self.page >= self.index:
        
            try:
                r = requests.get(self.link.format(self.index))
                d = json.loads(r.text)
                c = d['list']

                if len(c)>0:
                    self.content = self.content+c
                    #print '{}/{}'.format(self.index,self.page),
                    print 'O',
                    self.index+= 1
                    self.process()
                else:
                    print '<',
                    self.process()
            except Exception as e:
                #print e,
                print 'X',
                self.process()
        else:
            print 'R',
            if self.all==0:
                print '0e',
            elif len(self.content)==self.all:
                try:
                    self.df = pd.DataFrame(self.content)
                    print self.df.shape,
                    self.df.to_excel(folder+str(self.pid)+'.xlsx', index=False)
                    print str(self.pid)+'.xlsx Done ',
                except Exception as e:
                    print e,
                    print 'S',
            else:
                print 'gather', len(self.content), 'exists', self.all
    def __repr__(self):
        return str(self.pid)+' instance'
    
def helper(ID):
    c = Transaction(ID)
    c.initial()
    c.process()
    
    
    
    
    
def transaction_main():
    global link, folder
    
    try:
        sys.path.append(os.getcwd())
        from info import transaction as link
        print 'imported transaction link from info'
        sys.path.remove(os.getcwd())

    except Exception as e:
        print e
        print 'lack of info.py'
        exit(0)    
    
    
    ct = ctime().split()
    folder= ct[2]+ct[1]+ct[-1]+'_transaction/'
    try:
        os.mkdir(folder)
    except:
        print 'D'    
        
    # cancat to one excel after down
    flag = raw_input('merge into one excel? do this after done scraping!  y/n: ')
    if flag.lower()=='y':

        print 'start concating data'
        start2=time()        
        
        files = os.listdir(folder)
        print len(files)

        tmp = []
        for i in files:
            try:
                one = pd.read_excel(folder+i)
                one['PID']=int(i.split('.')[0])
                tmp.append(one)
            except:
                print i,
        print len(tmp)
        df = pd.concat(tmp)
        print df.shape        
    
        col = list(df.columns)
        change_order = ['PID',]
        col = [col.pop(col.index(i)) for i in change_order]+col
        df = df[col]

        end2 = time()
        elapse2 = end2 - start2 
        print 'used {:.2f} s, {:.2f} mins'.format(elapse2, elapse2/60)
        print 'saving to transactions.xlsx'
        df.to_excel(strftime("%Y-%m-%d-%H-%M",localtime())+ ' transaction.xlsx', encoding='utf-8', index=False)
        print 'done!', ctime()
#### start
    content = pd.read_csv('./transaction_products.csv')['pid'].tolist()
    cpu = int(raw_input('(multi-processing) how many process to run ? '))

    have = list(map(lambda x: int(x.split('.')[0]) ,os.listdir(folder)))
    want = list(set(content)-set(have))
    
    print 'all product diary - already scraped'
    print len(content), len(have), len(want)
    
    
    
    start=time()

    pool = ThreadPool(cpu)
    
    results = pool.map(helper, want)
    
    pool.close()
    pool.join()
    
    end = time()
    elapse = end - start 
    
    print 'used {:.2f} s, {:.2f} mins'.format(elapse, elapse/60)
        
    


# coding: utf-8



import sys
try:
    reload(sys)
    sys.setdefaultencoding("utf-8")
except:
    print('python3')
    
import pandas as pd
import json
import requests
import os
from time import time, ctime
from json2df import flatten_dict







class Collect(object):
    
    def __init__(self, link, place_id, cat_id):
        
        
        self.link =  link.format(place_id, cat_id, '{}')

        self.place_id = place_id
        self.cat_id = cat_id
        self.collect = []
        self.df = None
        self.flag = 0 
        
        
    def __scrape(self, index):
        
        try:
            products = json.loads(requests.get(self.link.format(index), timeout=3).text)['responseData']['product_info']
            print 'O',
            
            if len(products)==0:
                try: 
                    if 0==len(json.loads(requests.get(self.link.format(index), timeout=3).text)['responseData']['product_info']):
                        print 'E',
                        self.flag = 1
                        return
                except:
                    print '?',
                    self.__scrape(index)
            

            for product in products:
                # clean field "doctor" for encoding
                s = ''
                try:
                    for d in product['doctor']:
                        for e in d.items():
                            s = s+'::'.join(e)+'--'
                    product['doctor'] = s
                except:
                    print 'D',
                
                # clean field
                try:
                    product['wei_kuan'] = '::'.join(product['wei_kuan'])
                    product['wei_kuan_list'] = '::'.join(product['wei_kuan_list'])
                except:
                    print 'W',
                    
                try:
                    product = flatten_dict(product, layers=3)
                except:
                    print 'P',
                    
                self.collect.append(product)
                
        except Exception as e:
            print e
            print 'T',
            exit(0)
            self.__scrape(index)  
            
    def main(self):
        
            
        for i in range(180):
            if not self.flag:
                self.__scrape(i)
                    
                                    
        print(len(self.collect))

        self.df = pd.DataFrame(self.collect) 
        
        try:
            col = list(self.df.columns)
            col = [col.pop(col.index('pid')), col.pop(col.index('title'))]+col
            self.df = self.df[col]
        except:
            print 'U'
        
        return self.df     
    def save(self):
        
        ct = ctime().split()
        folder= ct[2]+ct[1]+ct[-1]+'_product/'
        try:
            os.mkdir(folder)
        except:
            print 'R'
        try:
            name = "{}_{}.xlsx".format(self.place_id, self.cat_id)
            self.df.to_excel('./'+folder + name, encoding="utf-8", index=False)
            print "Save Success! for "+name
        except:
            print "Save Error"
            
        print len(os.listdir(folder)),'files scraped!'
    def __repr__(self):
        return "place: "+ str(self.place_id)+ " category: "+ str(self.cat_id)+ " scraped: "+ str(self.flag==1)
        

